@@ -1,7 +1,5 @@
-#include "opencv2/core/utility.hpp"
-#include "opencv2/core/private.hpp"
 #include "opencv2/ts.hpp"
-#include "cvconfig.h"
+#include "opencv2/core/private.hpp"
 
 #ifdef GTEST_LINKED_AS_SHARED_LIBRARY
 #error ts module should not have GTEST_LINKED_AS_SHARED_LIBRARY defined
